@@ -8,7 +8,7 @@ if TYPE_CHECKING:
 
 
 class File:
-    def __init__(self, path: Path | None, contents: str = ''):
+    def __init__(self, path: Path | None, contents: str = ""):
         self.path = path
         self.contents = contents
         self.feature = None
@@ -19,7 +19,7 @@ class File:
 
         path = root / self.path
         path.ensure_parent_dir_exists()
-        path.write_text(self.contents, encoding='utf-8')
+        path.write_text(self.contents, encoding="utf-8")
 
 
 def find_template_files(module):
